/* Copyright (c) 2012 Nordic Semiconductor. All Rights Reserved.
 *
 * The information contained herein is property of Nordic Semiconductor ASA.
 * Terms and conditions of usage are described in detail in NORDIC
 * SEMICONDUCTOR STANDARD SOFTWARE LICENSE AGREEMENT.
 *
 * Licensees are granted free, non-transferable use of the information. NO
 * WARRANTY of ANY KIND is provided. This heading must NOT be removed from
 * the file.
 *
 */

/** @file
 *
 * @defgroup ble_sdk_app_template_main main.c
 * @{
 * @ingroup ble_sdk_app_template
 * @brief Template project main file.
 *
 * This file contains a template for creating a new application. It has the code necessary to wakeup from button, advertise, get a connection
 * restart advertising on disconnect and if no new connection created go back to system-off mode.
 * It can easily be used as a starting point for creating a new application, the comments identified with 'YOUR_JOB' indicates where
 * and how you can customize.
 */

#include <stdint.h>
#include <string.h>
#include "nordic_common.h"
#include "nrf.h"
#include "app_error.h"
#include "nrf_soc.h"
#include "nrf_gpio.h"
#include "nrf51_bitfields.h"
#include "ble_conn_params.h"
#include "ble_nrf6310_pins.h"
#include "ble_error_log.h"
#include "app_gpiote.h"
#include "ble_debug_assert_handler.h"

#define APP_GPIOTE_MAX_USERS            1                                           /**< Maximum number of users of the GPIOTE handler. */

static app_gpiote_user_id_t m_gpiote_user_id;

/**@brief Error handler function, which is called when an error has occurred. 
 *
 * @warning This handler is an example only and does not fit a final product. You need to analyze 
 *          how your product is supposed to react in case of error.
 *
 * @param[in] error_code  Error code supplied to the handler.
 * @param[in] line_num    Line number where the handler is called.
 * @param[in] p_file_name Pointer to the file name. 
 */
void app_error_handler(uint32_t error_code, uint32_t line_num, const uint8_t * p_file_name)
{
    nrf_gpio_pin_set(ASSERT_LED_PIN_NO);

    // This call can be used for debug purposes during development of an application.
    // @note CAUTION: Activating this code will write the stack to flash on an error.
    //                This function should NOT be used in a final product.
    //                It is intended STRICTLY for development/debugging purposes.
    //                The flash write will happen EVEN if the radio is active, thus interrupting
    //                any communication.
    //                Use with care. Un-comment the line below to use.
    ble_debug_assert_handler(error_code, line_num, p_file_name);

    // On assert, the system can only recover with a reset.
    //NVIC_SystemReset();
}


/**@brief Assert macro callback function.
 *
 * @details This function will be called in case of an assert in the SoftDevice.
 *
 * @warning This handler is an example only and does not fit a final product. You need to analyze 
 *          how your product is supposed to react in case of Assert.
 * @warning On assert from the SoftDevice, the system can only recover on reset.
 *
 * @param[in]   line_num   Line number of the failing ASSERT call.
 * @param[in]   file_name  File name of the failing ASSERT call.
 */
void assert_nrf_callback(uint16_t line_num, const uint8_t * p_file_name)
{
    app_error_handler(0xDEADBEEF, line_num, p_file_name);
}

/**@brief LEDs initialization.
 *
 * @details Initializes all LEDs used by the application.
 */
static void leds_init(void)
{
    nrf_gpio_range_cfg_output(8, 15);
}


static void gpiote_event_handler(uint32_t pins_low_to_high, uint32_t pins_high_to_low)
{
    nrf_gpio_pin_toggle(14);
    if (pins_high_to_low & (1 << NRF6310_BUTTON_0))
    {
        nrf_gpio_pin_set(8);
    }
    if (pins_high_to_low & (1 << NRF6310_BUTTON_1))
    {
        nrf_gpio_pin_set(9);
    }
    if (pins_low_to_high & (1 << NRF6310_BUTTON_0))
    {
        nrf_gpio_pin_clear(8);
    }
    if (pins_low_to_high & (1 << NRF6310_BUTTON_1))
    {
        nrf_gpio_pin_clear(9);
    }
}


/**@brief Initialize GPIOTE handler module.
 */
static void gpiote_init(void)
{
    nrf_gpio_range_cfg_input(0, 7, NRF_GPIO_PIN_NOPULL);
    
    APP_GPIOTE_INIT(APP_GPIOTE_MAX_USERS);
    
    uint32_t pin_mask = (1 << NRF6310_BUTTON_1) | (1 << NRF6310_BUTTON_0);
    uint32_t err_code = app_gpiote_user_register(&m_gpiote_user_id, pin_mask, pin_mask, gpiote_event_handler);
    APP_ERROR_CHECK(err_code);
    
    err_code = app_gpiote_user_enable(m_gpiote_user_id);
    APP_ERROR_CHECK(err_code);
}


/**@brief Application main function.
 */
int main(void)
{
    leds_init();
    gpiote_init();
 
    for (;;)
    {
        __WFI();
    }
}

/** 
 * @}
 */
